<?php

namespace Netzmacher\Deal\TCA;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2018-2019 - Dirk Wildt <http://wildt.at.die-netzmacher.de/>
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * Methods for the TCA
 *
 * @package deal
 * @license http://www.gnu.org/licenses/lgpl.html
 * 			GNU Lesser General Public License, version 3 or later
 * @version 7.3.1
 * @since 7.3.1
 */
class Tree
{

	/**
	 * SelectTree() : Renders a TCA selecttree
	 *
	 * @param string $label From a locallang.xlf
	 * @param string $foreign foreign_table
	 * @param string $where SQL-where-statement for table_foreign
	 * @param integer $size Size of the visible parte of the tree in lines
	 * @param boolean $expandAll expand the tree by default 
	 * @param string $parentField foreign_table
	 * 
	 * @return void
	 * @access public
	 * @version 7.3.1
	 * @since 7.3.1
	 */
	static function ParentTree( $label, $foreign_table, $where = NULL, $size = 20, $expandAll = false, $parentField = 'uid_parent' )
	{
		$return = array(
				'exclude' => 1,
				'label' => $label,
				'config' => array(
						'type' => 'select',
						'renderMode' => 'tree', // TYPO3 6.x
						'renderType' => 'selectTree', // TYPO3 7.6
						'foreign_table' => $foreign_table,
						// #i0157, 160918, 1+
						'foreign_table_where' => $where,
						'size' => $size,
						'treeConfig' => array(
								'parentField' => $parentField,
								'appearance' => array(
										'expandAll' => false,
										'showHeader' => true,
								),
						),
				),
		);

		if( $where )
		{
			$return[ 'config' ][ 'foreign_table_where' ] = $where;
		}

		return $return;
	}

	/**
	 * SelectTree() : Renders a TCA selecttree
	 *
	 * @param string $MMlocal table_local
	 * @param string $MMforeign table_foreign
	 * @param string $where SQL-where-statement for table_foreign
	 * @param integer $size Size of the visible parte of the tree in lines
	 * @param boolean $expandAll expand the tree by default 
	 * @param string $MM MM-table, the table for relations
	 * @param string $foreign foreign_table
	 * @param string $MMopposite Opposite field in case of an opposite relation.
	 * @param string $parentField foreign_table
	 * 
	 * @return void
	 * @access public
	 * @version 7.3.1
	 * @since 7.3.1
	 */
	static function SelectTree( $MMlocal, $MMforeign, $where = NULL, $size = 20, $expandAll = false, $MM = 'tx_quickshop_mm', $foreign = NULL, $MMopposite = NULL, $parentField = 'uid_parent' )
	{
		if( !$foreign )
		{
			$foreign = $MMforeign;
		}
		$return = array(
				'behaviour' => array(
						'allowLanguageSynchronization' => true,
				),
				'foreign_table' => $foreign,
				'foreign_table_where' => $where,
				'maxitems' => 9999,
				'MM' => $MM,
				"MM_match_fields" => array(
						'table_local' => $MMlocal,
						'table_foreign' => $MMforeign,
				),
				"MM_insert_fields" => array(
						'table_local' => $MMlocal,
						'table_foreign' => $MMforeign,
				),
				'renderMode' => 'tree', // TYPO3 6.x
				'renderType' => 'selectTree', // TYPO3 7.x
				'size' => $size,
				'treeConfig' => array(
						'parentField' => $parentField,
						'appearance' => array(
								'expandAll' => $expandAll,
								'showHeader' => true,
						),
				),
				'type' => 'select',
		);

		if( $MMopposite )
		{
			$return[ 'MM_opposite_field' ] = $MMopposite;
		}

		return $return;
	}

}
